@@ -512,7 +512,7 @@ void GkStatus::AuthenticateClient(
 			<< ", login: " << newClient->GetUser()
 			);
 		newClient->WriteString("\r\nAccess forbidden!\r\n");
-		newClient->Flush();
+		// newClient->Flush();	// dont' flush when access is denied to avoid blocking
 		delete newClient;
 	}
 }
@@ -712,6 +712,7 @@ void GkStatus::OnStart()
 	m_commands["rta"] = e_RouteToAlias;
 	m_commands["routetogateway"] = e_RouteToGateway;
 	m_commands["rtg"] = e_RouteToGateway;
+	m_commands["bindandroutetogateway"] = e_BindAndRouteToGateway;
 	m_commands["routereject"] = e_RouteReject;
 	m_commands["shutdown"] = e_Shutdown;
 	m_commands["exit"] = e_Exit;
@@ -737,6 +738,7 @@ void GkStatus::OnStart()
 	m_commands["gai"] = e_GetAuthInfo;
 	m_commands["getacctinfo"] = e_GetAcctInfo;
 	m_commands["gci"] = e_GetAcctInfo;
+	m_commands["resetcallcounters"] = e_ResetCallCounters;
 }
 
 void GkStatus::ReadSocket(
@@ -784,16 +786,14 @@ StatusClient::StatusClient(
         PStringToString filters = GkConfig()->GetAllKeyValues(filteringsec);
         SetWriteTimeout(10);
 	
-	PString key;
-	PString data;
 	for (PINDEX i = 0; i < filters.GetSize(); i++) {
-	    key = filters.GetKeyAt(i);
-	    data = filters.GetDataAt(i);
+	    PString key = filters.GetKeyAt(i);
+	    PString data = filters.GetDataAt(i);
 	    PStringArray regexArray(data.Tokenise("\n", false));
 	    for (PINDEX k = 0; k < regexArray.GetSize(); ++k) {
-		if(filters.GetKeyAt(i) == "IncludeFilter")
+		if (filters.GetKeyAt(i) == "IncludeFilter")
 		    AddFilter(m_includeFilterRegex, regexArray[k]);
-		else if(filters.GetKeyAt(i) == "ExcludeFilter")
+		else if (filters.GetKeyAt(i) == "ExcludeFilter")
 		    AddFilter(m_excludeFilterRegex, regexArray[k]);
 	    }
 	}
@@ -1194,7 +1194,7 @@ void StatusClient::ExecCommand(
 		break;
 	case GkStatus::e_PrintCurrentCalls:
 		// print list of currently ongoing calls
-		SoftPBX::PrintCurrentCalls(this);
+		SoftPBX::PrintCurrentCalls(this, FALSE);
 		break;
 	case GkStatus::e_PrintCurrentCallsVerbose:
 		// print list of currently ongoing calls
@@ -1203,6 +1203,9 @@ void StatusClient::ExecCommand(
 	case GkStatus::e_Statistics:
 		SoftPBX::PrintStatistics(this, TRUE);
 		break;
+	case GkStatus::e_ResetCallCounters:
+		SoftPBX::ResetCallCounters(this);
+		break;
 	case GkStatus::e_Find:
 		if (args.GetSize() == 2)
 			SoftPBX::PrintEndpoint(args[1], this, FALSE);
@@ -1345,6 +1348,16 @@ void StatusClient::ExecCommand(
 		} else
 			CommandError("Syntax Error: RouteToGateway TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID]");
 		break;
+	case GkStatus::e_BindAndRouteToGateway:
+		if (args.GetSize() == 6) {
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), "", args[1]);
+		} else if (args.GetSize() == 7) {
+			args[6].Replace("-", " ", true);
+			args[6] = args[6].Trim();
+			RasServer::Instance()->GetVirtualQueue()->RouteToAlias(args[2], args[3], args[4], args[5].AsUnsigned(), args[6], args[1]);
+		} else
+			CommandError("Syntax Error: BindAndRouteToGateway BIND_IP TARGET_ALIAS TARGET_IP CALLING_ENDPOINT_ID CRV [CALLID]");
+		break;
 	case GkStatus::e_RouteReject:
 		if (args.GetSize() == 3) {
 			RasServer::Instance()->GetVirtualQueue()->RouteReject(args[1], args[2].AsUnsigned(), "");
