@@ -10,7 +10,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASTBL_H
-#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.155 2009/06/02 14:23:31 willamowius Exp $"
+#define RASTBL_H "@(#) $Id: RasTbl.h,v 1.174 2010/05/03 12:38:33 shorne Exp $"
 
 #include <list>
 #include <map>
@@ -58,7 +58,7 @@ private:
 	T *pt;
 };
 
-#ifdef hasPresence
+#ifdef HAS_H460P
 class H323Contact;
 #endif
 class EndpointRec
@@ -98,6 +98,7 @@ public:
 	virtual void SetEndpointIdentifier(const H225_EndpointIdentifier &);
 	virtual void SetTimeToLive(int);
 	virtual void SetAliases(const H225_ArrayOf_AliasAddress &);
+	virtual void AddNumbers(const PString & numbers);
 	virtual void SetEndpointType(const H225_EndpointType &);
     virtual void SetEndpointInfo(const PString & vendor, const PString & version);
 
@@ -142,7 +143,9 @@ public:
 	void SetNAT(bool nat);
 	void SetNATAddress(const PIPSocket::Address &);
 	void SetSocket(CallSignalSocket *);
-	void SetSupportNAT(bool support);
+	void SetH46024(bool support);
+	void SetH46024A(bool support);
+	void SetH46024B(bool support);
 
 	enum EPNatTypes {
             NatUnknown,
@@ -159,6 +162,7 @@ public:
 	void SetEPNATType(int nattype) {m_epnattype = (EPNatTypes)nattype; }
 	void SetNATProxy(PBoolean support) {m_natproxy = support; }
 	void SetInternal(PBoolean internal) { m_internal = internal; }
+	void SetUsesH46023(bool uses) { m_usesH46023 = uses; }
 
 	void SetPriority(int priority) { m_registrationPriority = priority; }
 	void SetPreemption(bool support) { m_registrationPreemption = support; }
@@ -173,25 +177,29 @@ public:
 	bool IsUsed() const;
 	bool IsUpdated(const PTime *) const;
 	bool IsNATed() const;
-	bool SupportNAT() const;
+	bool SupportH46024() const;
+	bool SupportH46024A() const;
+	bool SupportH46024B() const;
+	bool UseH46024B() const;
+	bool UsesH46023() const { return m_usesH46023; }
 
 	bool HasNATProxy() const;
 	bool IsInternal() const;
 	bool IsRemote() const;
-	int GetEPNATType() {return (int)m_epnattype; }
+	int GetEPNATType() const { return (int)m_epnattype; }
 	static PString GetEPNATTypeString(EPNatTypes nat);
 	bool SupportPreemption() const { return m_registrationPreemption; }
-	H225_AlternateGK GetAssignedGatekeeper() { return m_assignedGatekeeper; }
+	H225_AlternateGK GetAssignedGatekeeper() const { return m_assignedGatekeeper; }
 
 	int  Priority() const { return m_registrationPriority; }
 	bool HasNATSocket() const;
 	PTime GetUpdatedTime() const;
 
-#ifdef hasPresence
-	bool hasContact();
-	void CreateContact();
-	void ParsePresencePDU(unsigned msgtag, const H225_EndpointIdentifier * id, const H225_FeatureDescriptor & pdu);
-	void BuildPresencePDU(unsigned msgtag, H225_FeatureDescriptor & pdu);
+	void SetUsesH460P(bool uses);
+	bool UsesH460P() const { return m_usesH460P; }
+#ifdef HAS_H460P
+	void ParsePresencePDU(const PASN_OctetString & pdu);
+	bool BuildPresencePDU(unsigned msgtag, PASN_OctetString & pdu);
 #endif
 
 	/** If this Endpoint would be register itself again with all the same data
@@ -243,7 +251,7 @@ public:
 	*/
 	bool HasAvailableCapacity(const H225_ArrayOf_AliasAddress & aliases) const;
 	// void DumpPrefixCapacity() const;
-	string LongestPrefixMatch(const PString & alias, int * capacity) const;
+	string LongestPrefixMatch(const PString & alias, int & capacity) const;
 	void UpdatePrefixStats(const PString & dest, int update);
 
 	// cause code translation
@@ -327,13 +335,11 @@ protected:
 	std::map<unsigned, unsigned> m_sentCauseMap;
 
 	EPNatTypes m_epnattype;
-	bool m_natsupport, m_natproxy, m_internal, m_remote;
+	bool m_usesH46023, m_H46024, m_H46024a, m_H46024b, m_natproxy, m_internal, m_remote;
 	bool m_h46018disabled;
 	bool m_usesH46018;
+	bool m_usesH460P;
 
-#ifdef hasPresence
-	H323Contact * m_contact;
-#endif
 };
 
 typedef EndpointRec::Ptr endptr;
@@ -453,6 +459,7 @@ public:
 
 	endptr FindByEndpointId(const H225_EndpointIdentifier & endpointId) const;
 	endptr FindBySignalAdr(const H225_TransportAddress &, PIPSocket::Address = INADDR_ANY) const;
+	endptr FindBySignalAdrIgnorePort(const H225_TransportAddress &, PIPSocket::Address = INADDR_ANY) const;
 	endptr FindOZEPBySignalAdr(const H225_TransportAddress &) const;
 	endptr FindByAliases(const H225_ArrayOf_AliasAddress & alias) const;
 	endptr FindFirstEndpoint(const H225_ArrayOf_AliasAddress & alias);
@@ -529,8 +536,11 @@ private:
 template<class> class RasPDU;
 
 // record of one active call
-#ifdef hasH460
+#ifdef HAS_H460
  class H4609_QosMonitoringReportData;
+#ifdef HAS_H46024B
+ class H323TransportAddress;
+#endif
 #endif
 class CallRec {
 public:
@@ -541,6 +551,14 @@ public:
 		ProxyDisabled /// disable full proxy mode
 	};
 
+	/// the combination of ProxyMode, RoutedMode and H245 routing
+	enum RoutingMode {
+		Undefined = 0,
+		SignalRouted,
+		H245Routed,
+		Proxied
+	};
+
 	/// who disconnected the call
 	enum ReleaseSource {
 		ReleasedByGatekeeper,
@@ -608,6 +626,7 @@ public:
 		PIPSocket::Address& calledPartyNATIP
 		) const;
 
+#ifdef HAS_H46023
 	// Override the calculated NAT type
 	void SetNATType(int newNatType) { m_nattype = newNatType; }
 
@@ -621,7 +640,8 @@ public:
 		e_natLocalProxy,
 	    e_natRemoteProxy,
 		e_natFullProxy,
-		e_natSameNAT,
+		e_natAnnexA,		// Same NAT
+		e_natAnnexB,		// Nat Offload
 		e_natFailure = 100
 	};
 
@@ -632,12 +652,29 @@ public:
 	/** Get String representation of the NATStrategy */
 	PString GetNATOffloadString(NatStrategy nat);
 
+
 	/** Get the NATStrategy Default is Unknown */
-	NatStrategy GetNATStrategy() { return m_natstrategy; }
+	NatStrategy GetNATStrategy() const { return m_natstrategy; }
 
 	/** Set the NATStrategy */
 	void SetNATStrategy(NatStrategy strategy) { m_natstrategy = strategy; }
 
+	/** Can Signalling be offloaded
+	  */
+	bool NATSignallingOffload(bool isAnswer);
+
+#ifdef HAS_H46024B
+	/** Initiate Probe */
+	void H46024BInitiate(WORD sessionID, const H323TransportAddress & fwd, const H323TransportAddress & rev);
+
+	/** Response Probe */
+	void H46024BRespond();
+
+    /** Set session callback flag */
+	void H46024BSessionFlag(WORD sessionID);
+#endif
+
+#endif
 
 	/** Return whether the endpoints are registered at the same gatekeeper so
 	    only 1 gatekeeper is involved in the call
@@ -690,13 +727,14 @@ public:
 	bool CompareCallNumber(PINDEX CallNumber) const;
 	bool CompareEndpoint(const endptr *) const;
 	bool CompareSigAdr(const H225_TransportAddress *adr) const;
+	bool CompareSigAdrIgnorePort(const H225_TransportAddress *adr) const;
 
 	bool IsUsed() const { return (m_usedCount != 0); }
 
 	/** @return
 		true if the call has been connected - a Connect message
-		has been received in gk routed signalling or the call has been admitted
-		(ARQ->ACF) in direct signalling. Does not necessary mean
+		has been received in gk routed signaling or the call has been admitted
+		(ARQ->ACF) in direct signaling. Does not necessary mean
 		that the call is still in progress (may have been already disconnected).
 	*/
 	bool IsConnected() const { return (m_connectTime != 0); }
@@ -746,6 +784,10 @@ public:
 		long seconds /// duration limit to be set
 		);
 
+	/** Set the client provided authentication id **/
+	void SetClientAuthId(PUInt64 id) { m_clientAuthId = id; }
+	PUInt64 GetClientAuthId() const { return m_clientAuthId; }
+
 	/** Set list of codecs to disable for this call */
 	void SetDisabledCodecs(const PString & codecs);
 	PString GetDisabledCodecs() const { return m_disabledcodecs; }
@@ -756,6 +798,63 @@ public:
 		has not been yet received.
 		Meaningful only in GK routed mode.
 	*/
+
+	PString GetSRC_media_control_IP() const;
+        PString GetDST_media_control_IP() const;
+    
+        void SetSRC_media_control_IP(PString IP);
+        void SetDST_media_control_IP(PString IP);
+    
+        PString GetSRC_media_IP() const; 
+        PString GetDST_media_IP() const; 
+                                             
+        void SetSRC_media_IP(PString IP);
+        void SetDST_media_IP(PString IP);
+    
+        void SetRTCP_SRC_sdes(PString val);
+        void SetRTCP_DST_sdes(PString val);
+    
+        PStringList GetRTCP_SRC_sdes() const;
+        PStringList GetRTCP_DST_sdes() const;
+    
+        bool GetRTCP_SRC_sdes_flag() const;
+        bool GetRTCP_DST_sdes_flag() const;
+	
+        void InitRTCP_report();
+    
+        void SetRTCP_SRC_packet_count(long val);
+        void SetRTCP_DST_packet_count(long val);
+    
+        void SetRTCP_SRC_packet_lost(long val);
+        void SetRTCP_DST_packet_lost(long val);
+    
+        void SetRTCP_SRC_jitter(int val);
+    
+        void SetRTCP_DST_jitter(int val);
+
+        // Get RTCP source packet count
+        long GetRTCP_SRC_packet_count() const;
+        // Get RTCP destination packet count
+        long GetRTCP_DST_packet_count() const;
+    
+        // Get RTCP source packet lost
+        long GetRTCP_SRC_packet_lost() const;
+        // Get RTCP destination packet lost
+        long GetRTCP_DST_packet_lost() const;
+    
+        // Get RTCP source jitter max
+        int GetRTCP_SRC_jitter_max() const;
+        // Get RTCP source jitter min
+        int GetRTCP_SRC_jitter_min() const;
+        // Get RTCP source jitter avg
+        int GetRTCP_SRC_jitter_avg() const;
+    
+        //Get RTCP destinaton jitter max
+        int GetRTCP_DST_jitter_max() const;
+        //Get RTCP destinaton jitter in
+        int GetRTCP_DST_jitter_min() const;
+        //Get RTCP destinaton jitter avg
+        int GetRTCP_DST_jitter_avg() const;
 	time_t GetSetupTime() const;
 
 	/** Set timestamp for a Setup message associated with this call. */
@@ -817,7 +916,7 @@ public:
 	}
 
 	/** Check if:
-		- a signalling channel associated with this call is not timed out
+		- a signaling channel associated with this call is not timed out
 		  and the call should be disconnected (removed from CallTable);
 		- call duration limit has been exceeded
 		- call should be disconnected from other reason
@@ -868,26 +967,26 @@ public:
 	*/
 	PString GetSrcInfo() const { return m_srcInfo; }
 
-	/** Set a new address for the calling party signalling channel.
+	/** Set a new address for the calling party signaling channel.
 	*/
 	void SetSrcSignalAddr(
-		const H225_TransportAddress & addr /// new signalling transport address
+		const H225_TransportAddress & addr /// new signaling transport address
 		);
 
-	/** Set the unregistered calling party signalling channel as NATed.
+	/** Set the unregistered calling party signaling channel as NATed.
 	*/
 	void SetSrcNATed(const PIPSocket::Address & natip);
 
-	/** Set a new address for the called party signalling channel.
+	/** Set a new address for the called party signaling channel.
 	*/
 	void SetDestSignalAddr(
-		const H225_TransportAddress & addr /// new signalling transport address
+		const H225_TransportAddress & addr /// new signaling transport address
 		);
 
 	H225_TransportAddress GetSrcSignalAddr() const;
 
 	/** Get IP and port for the calling party. It is a signal address
-		for registered endpoints and remote signalling socket address
+		for registered endpoints and remote signaling socket address
 		for unregistered endpoints.
 
 		@return
@@ -901,7 +1000,7 @@ public:
 	H225_TransportAddress GetDestSignalAddr() const;
 
 	/** Get IP and port for the called party. It is a signal address
-		for registered endpoints and remote signalling socket address
+		for registered endpoints and remote signaling socket address
 		for unregistered endpoints.
 
 		@return
@@ -1028,12 +1127,17 @@ public:
 	PBYTEArray GetRADIUSClass() const;
 	
 	bool IsProceedingSent() const { return m_proceedingSent; }
-	// TODO: JW also set when sent via status port
+	// TODO: also set when sent through a status port command
 	void SetProceedingSent(bool val) { m_proceedingSent = val; }
 	
+	void SetBindHint(const PString & ip) { m_bindHint = ip; }
+	PString GetBindHint() const { return m_bindHint; }
+	
 #ifdef HAS_H46018
+	bool H46019Required();
 	void StoreSetup(SignalingMsg * msg);
 	PBYTEArray RetrieveSetup();
+	int GetH46019Direction();
 #endif
 
 private:
@@ -1078,6 +1182,42 @@ private:
 	
 	/// list of disabled codes
 	PString m_disabledcodecs;
+	
+	PString m_src_media_control_IP, m_dst_media_control_IP;
+	PString m_src_media_IP, m_dst_media_IP;
+
+	PStringList m_rtcp_source_sdes;
+	bool m_rtcp_source_sdes_flag;
+
+	PStringList m_rtcp_destination_sdes;  
+	bool m_rtcp_destination_sdes_flag;
+
+	// RTCP_source_packet_count
+	long m_rtcp_source_packet_count;
+	// RTCP_destination_packet_count	
+	long m_rtcp_destination_packet_count;
+
+	// RTCP_source_packet_lost
+	long m_rtcp_source_packet_lost;
+	// RTCP_destination_packet_lost
+	long m_rtcp_destination_packet_lost;
+
+	// RTCP_source_jitter
+	int m_rtcp_source_jitter_min;
+	int m_rtcp_source_jitter_max;
+	int m_rtcp_source_jitter_avg;
+
+	// RTCP_destination_jitter
+	int m_rtcp_destination_jitter_min;
+	int m_rtcp_destination_jitter_max;
+	int m_rtcp_destination_jitter_avg;
+
+
+	int m_rtcp_source_jitter_avg_count;
+	long m_rtcp_source_jitter_avg_sum;
+
+	int m_rtcp_destination_jitter_avg_count;
+	long m_rtcp_destination_jitter_avg_sum;
 
 	/// current timeout (or duration limit) for the call
 	time_t m_timeout;
@@ -1107,9 +1247,9 @@ private:
 	int  m_releaseSource;
 	/// unique accounting session id associated with this call
 	PString m_acctSessionId;
-	/// signalling transport address of the calling party
+	/// signaling transport address of the calling party
 	H225_TransportAddress m_srcSignalAddress;
-	/// signalling transport address of the called party
+	/// signaling transport address of the called party
 	H225_TransportAddress m_destSignalAddress;
 	/// calling party's number
 	PString m_callingStationId;
@@ -1127,8 +1267,21 @@ private:
 	int m_usedCount;
 	mutable PTimedMutex m_usedLock, m_sockLock;
 	int m_nattype;
+#ifdef HAS_H46023
 	NatStrategy m_natstrategy;
+#endif
 
+#ifdef HAS_H46024B
+	struct H46024Balternate {
+		 H245_TransportAddress forward;
+		 H245_TransportAddress reverse;
+	};
+
+	std::map<WORD,H46024Balternate> m_H46024Balternate;
+	void BuildH46024AnnexBMessage(bool initiate,H245_MultimediaSystemControlMessage & h245msg, const map<WORD,H46024Balternate> & alt);
+	list<int> m_h46024Bflag;
+	PMutex m_H46024Bmutex;
+#endif
 	/// unregistered caller NAT'd
 	bool m_unregNAT;
 	PIPSocket::Address m_srcunregNATAddress;
@@ -1165,6 +1318,8 @@ private:
 	/// processed Setup data ready to be sent to the callee (for H.460.18)
 	PBYTEArray m_processedSetup;
 #endif
+	PUInt64 m_clientAuthId;
+	PString m_bindHint;	// outgoing IP or empty
 };
 
 typedef CallRec::Ptr callptr;
@@ -1181,6 +1336,7 @@ public:
 	CallTable();
 	~CallTable();
 
+	void ResetCallCounters();
 	void Insert(CallRec * NewRec);
 
 	// bandwidth management
@@ -1194,6 +1350,7 @@ public:
 	callptr FindCallRec(PINDEX CallNumber) const;
 	callptr FindCallRec(const endptr &) const;
 	callptr FindBySignalAdr(const H225_TransportAddress & SignalAdr) const;
+	callptr FindBySignalAdrIgnorePort(const H225_TransportAddress & SignalAdr) const;
 
 	void ClearTable();
 	void CheckCalls(
@@ -1207,7 +1364,7 @@ public:
 	void PrintCurrentCalls(USocket *client, bool verbose=FALSE) const;
 	PString PrintStatistics() const;
 
-#ifdef hasH460
+#ifdef HAS_H460
 	void OnQosMonitoringReport(const PString &,const endptr &, H4609_QosMonitoringReportData &);
     void QoSReport(const H225_DisengageRequest &, const endptr &, const PASN_OctetString &);
     void QoSReport(const H225_InfoRequestResponse &, const callptr &, const endptr &, const PASN_OctetString &);
@@ -1219,8 +1376,8 @@ public:
 	PINDEX Size() const { return m_activeCall; }
 
 	/** @return
-	    Timeout value for a signalling channel to be opened after ACF
-	    and for an Alerting message to be received after signalling start.
+	    Timeout value for a signaling channel to be opened after ACF
+	    and for an Alerting message to be received after signaling start.
 	    The value is expressed in milliseconds.
 	*/
 	long GetSignalTimeout() const { return m_signalTimeout; }
@@ -1269,7 +1426,7 @@ private:
 	unsigned m_CallCount, m_successCall, m_neighborCall, m_parentCall, m_activeCall;
 
 	/// timeout for a Connect message to be received
-	/// and for a signalling channel to be opened after ACF/ARQ
+	/// and for a signaling channel to be opened after ACF/ARQ
 	/// (0 if GK is not in routed mode)
 	long m_signalTimeout;
 	/// timeout for a Connect message to be received after getting an Alerting message
@@ -1328,9 +1485,9 @@ inline H225_EndpointIdentifier EndpointRec::GetEndpointIdentifier() const
   
 inline int EndpointRec::GetTimeToLive() const
 {
-	if (UsesH46018()) {
-		// force timeToLive to 5 - 30 sec, 29 sec if not set
-		return m_timeToLive == 0 ? 29 : max(5, min(30, m_timeToLive));
+	if (m_nat || UsesH46018()) {
+		// force timeToLive to 5 - 30 sec, 19 sec if not set
+		return m_timeToLive == 0 ? 19 : max(5, min(30, m_timeToLive));
 	}
 	return m_timeToLive;
 }
@@ -1382,9 +1539,19 @@ inline void EndpointRec::SetNAT(bool nat)
 	m_nat = nat;
 }
 
-inline void EndpointRec::SetSupportNAT(bool support)
+inline void EndpointRec::SetH46024(bool support)
 {
-	m_natsupport = support;
+	m_H46024 = support;
+}
+
+inline void EndpointRec::SetH46024A(bool support)
+{
+	m_H46024a = support;
+}
+
+inline void EndpointRec::SetH46024B(bool support)
+{
+	m_H46024b = support;
 }
 
 inline bool EndpointRec::IsNATed() const
@@ -1392,9 +1559,25 @@ inline bool EndpointRec::IsNATed() const
 	return m_nat;
 }
 
-inline bool EndpointRec::SupportNAT() const
+inline bool EndpointRec::SupportH46024() const
 {
-	return m_natsupport;
+	return m_H46024;
+}
+
+inline bool EndpointRec::SupportH46024A() const
+{
+	return m_H46024a;
+}
+
+inline bool EndpointRec::SupportH46024B() const
+{
+	return m_H46024b;
+}
+
+inline bool EndpointRec::UseH46024B() const
+{
+	int nat = (int)m_epnattype;
+	return (m_H46024b && (nat > 2 && nat < 5));
 }
 
 inline bool EndpointRec::HasNATProxy() const
@@ -1558,6 +1741,96 @@ inline bool CallRec::CompareSigAdr(const H225_TransportAddress *adr) const
 		(m_Called && m_Called->GetCallSignalAddress() == *adr);
 }
 
+inline PString CallRec::GetSRC_media_control_IP() const
+{
+    return m_src_media_control_IP;
+}
+
+inline PString CallRec::GetDST_media_control_IP() const
+{
+    return m_dst_media_control_IP;
+}
+
+inline PString CallRec::GetSRC_media_IP() const
+{
+    return m_src_media_IP;
+}
+
+inline PString CallRec::GetDST_media_IP() const
+{
+    return m_dst_media_IP;
+}
+
+inline bool CallRec::GetRTCP_SRC_sdes_flag() const
+{
+    return m_rtcp_source_sdes_flag;
+}
+
+inline bool CallRec::GetRTCP_DST_sdes_flag() const
+{
+    return m_rtcp_destination_sdes_flag;
+}
+
+inline PStringList CallRec::GetRTCP_SRC_sdes() const
+{
+    return m_rtcp_source_sdes;
+}
+
+inline PStringList CallRec::GetRTCP_DST_sdes() const
+{
+    return m_rtcp_destination_sdes;
+}
+
+inline long CallRec::GetRTCP_SRC_packet_count() const
+{
+    return m_rtcp_source_packet_count;
+}
+
+inline long CallRec::GetRTCP_DST_packet_count() const
+{
+    return m_rtcp_destination_packet_count;
+}
+
+inline long CallRec::GetRTCP_SRC_packet_lost() const
+{
+    return m_rtcp_source_packet_lost;
+}
+
+inline long CallRec::GetRTCP_DST_packet_lost() const
+{
+    return m_rtcp_destination_packet_lost;
+}
+
+inline int CallRec::GetRTCP_SRC_jitter_max() const
+{
+    return m_rtcp_source_jitter_max;
+}
+
+inline int CallRec::GetRTCP_SRC_jitter_min() const
+{
+    return m_rtcp_source_jitter_min;
+}
+
+inline int CallRec::GetRTCP_SRC_jitter_avg() const
+{
+    return m_rtcp_source_jitter_avg;
+}
+
+inline int CallRec::GetRTCP_DST_jitter_max() const
+{
+    return m_rtcp_destination_jitter_max;
+}
+
+inline int CallRec::GetRTCP_DST_jitter_min() const
+{
+    return m_rtcp_destination_jitter_min;
+}
+
+inline int CallRec::GetRTCP_DST_jitter_avg() const
+{
+    return m_rtcp_destination_jitter_avg;
+}
+
 inline time_t CallRec::GetSetupTime() const
 {
 	return m_setupTime;
