@@ -585,6 +585,9 @@ bool DNSPolicy::FindByAliases(
 	)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
+		// don't apply DNS to dialedDigits
+		if (aliases[i].GetTag() == H225_AliasAddress::e_dialedDigits)
+			continue;
 		PString alias(AsString(aliases[i], FALSE));
 		PINDEX at = alias.Find('@');
 
@@ -594,22 +597,72 @@ bool DNSPolicy::FindByAliases(
 			PIPSocket::Address addr;
 			if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
 				continue;
+			if (Toolkit::Instance()->IsGKHome(addr)) {
+				// check if the domain is my IP, if so route to local endpoint if available
+				H225_ArrayOf_AliasAddress find_aliases;
+				find_aliases.SetSize(1);
+				H323SetAliasAddress(alias.Left(at), find_aliases[0]);
+				endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+				if (ep) {
+					dest = ep->GetCallSignalAddress();
+				} else {
+					continue;	// can't route this alias locally, try next alias
+				}
+			} else {
+			}
 			Route route(m_name, dest);
 			route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(dest);
 			request.AddRoute(route);
-			request.SetFlag(RoutingRequest::e_aliasesChanged);
-			// remove the domain name part
+			// remove the domain name part (should not be necessary with latest GnuGk as destination, but keep for older ones)
 			H323SetAliasAddress(alias.Left(at), aliases[i]);
+			request.SetFlag(RoutingRequest::e_aliasesChanged);
+			PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at) << " @ " << AsDotString(dest));
 			return true;
 		}
 	}
 	return false;
 }
 
-bool DNSPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
-{ 
-	PTRACE(4, "ROUTING\tPolicy DNS not supported for LRQ");
-	return false; // DNSPolicy::FindByAliases((RoutingRequest&)request, aliases);
+bool DNSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
+{
+	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
+		// don't apply DNS to dialedDigits
+		if (aliases[i].GetTag() == H225_AliasAddress::e_dialedDigits)
+			continue;
+		PString alias(AsString(aliases[i], FALSE));
+		PINDEX at = alias.Find('@');
+
+		PString domain = (at != P_MAX_INDEX) ? alias.Mid(at + 1) : alias;
+		H225_TransportAddress dest;
+		if (GetTransportAddress(domain, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
+			PIPSocket::Address addr;
+			if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
+				continue;
+			if (Toolkit::Instance()->IsGKHome(addr)) {
+				// only apply DNS policy to LRQs that resolve locally
+				H225_ArrayOf_AliasAddress find_aliases;
+				find_aliases.SetSize(1);
+				H323SetAliasAddress(alias.Left(at), find_aliases[0]);
+				endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+				if (ep) {
+					if (!(RasServer::Instance()->IsGKRouted())) {
+						// in direct mode, send call directly to EP
+						dest = ep->GetCallSignalAddress();
+					}
+					Route route(m_name, dest);
+					route.m_destEndpoint = ep;
+					request.AddRoute(route);
+					request.SetFlag(RoutingRequest::e_aliasesChanged);
+					// remove the domain name part
+					H323SetAliasAddress(alias.Left(at), aliases[i]);
+					PTRACE(4, "ROUTING\tDNS policy resolves to " << alias.Left(at));
+					return true;
+				}
+			}
+		}
+	}
+	PTRACE(4, "ROUTING\tPolicy DNS only supports LRQs that resolve locally");
+	return false;
 }
 
 
@@ -712,6 +765,8 @@ bool VirtualQueue::SendRouteRequest(
 	/// destination (virtual queue) aliases as specified
 	/// by the calling endpoint (modified by this function on successful return)
 	PString* callSigAdr,
+	/// bind IP for BindAndRouteToGateway
+	PString* bindIP,
 	/// should the call be rejected modified by this function on return)
 	bool & reject,
 	/// actual virtual queue name (should be present in destinationInfo too)
@@ -727,14 +782,12 @@ bool VirtualQueue::SendRouteRequest(
 {
 	bool result = false;
 	bool duprequest = false;
-	if (RouteRequest *r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, duprequest)) {
-		PString msg = PString(PString::Printf, "RouteRequest|%s|%s|%u|%s|%s", 
-				(const char *)source,
-				(const char *)epid,
-				crv,
-				(const char *)vqueue,
-				(const char *)sourceInfo
-			   );
+	if (RouteRequest *r = InsertRequest(epid, crv, callID, destinationInfo, callSigAdr, bindIP, duprequest)) {
+		PString msg = "RouteRequest|" + source
+						+ "|" + epid
+						+ "|" + PString(crv)
+						+ "|" + vqueue
+						+ "|" + sourceInfo;
 		PString cid = callID;
 		cid.Replace(" ", "-", true);
 		msg += PString("|") + cid;
@@ -793,12 +846,14 @@ bool VirtualQueue::RouteToAlias(
 	unsigned crv,
 	/// callID of the call associated with the route request
 	const PString& callID,
+	// outgoing IP or empty
+	const PString& bindIP,
 	/// should this call be rejected
 	bool reject
 	)
 {
 	PWaitAndSignal lock(m_listMutex);
-	
+
 	// signal the command to each pending request
 	bool foundrequest = false;
 	RouteRequests::iterator i = m_pendingRequests.begin();
@@ -818,6 +873,7 @@ bool VirtualQueue::RouteToAlias(
 					*(r->m_agent) = agent;
 				if (!destinationip.IsEmpty())	// RouteToGateway
 					*(r->m_callsignaladdr) = destinationip;
+				*(r->m_sourceIP) = bindIP;	// BindAndRouteToGateway
 			}
 			r->m_reject = reject;
 			r->m_sync.Signal();
@@ -837,7 +893,7 @@ bool VirtualQueue::RouteToAlias(
 	
 	if( !foundrequest ) {
 		PTRACE(4, "VQueue\tPending route request (EPID:" << callingEpId
-			<< ", CRV=" << crv << ") not found - ignoring RouteToAlias / RouteToGateway command");
+			<< ", CRV=" << crv << ") not found - ignoring RouteToAlias / RouteToGateway / BindAndRouteToGateway command");
 	}
 	
 	return foundrequest;
@@ -855,6 +911,8 @@ bool VirtualQueue::RouteToAlias(
 	unsigned crv,
 	/// callID of the call associated with the route request
 	const PString& callID,
+	// outgoing IP or empty
+	const PString& bindIP,
 	/// should this call be rejected
 	bool reject
 	)
@@ -864,7 +922,7 @@ bool VirtualQueue::RouteToAlias(
 		alias.SetSize(1);
 		H323SetAliasAddress(targetAlias, alias[0]);
 	}
-	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID, reject);
+	return RouteToAlias(alias, destinationIp, callingEpId, crv, callID, bindIP, reject);
 }
 
 bool VirtualQueue::RouteReject(
@@ -877,7 +935,7 @@ bool VirtualQueue::RouteReject(
 	)
 {
 	H225_ArrayOf_AliasAddress nullAgent;
-	return RouteToAlias(nullAgent, "", callingEpId, crv, callID, true);
+	return RouteToAlias(nullAgent, "", callingEpId, crv, callID, "", true);
 }
 
 VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
@@ -893,6 +951,8 @@ VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
 	/// a pointer to a string  to be filled with a destinationCallSignalAddress
 	/// when the routing decision has been made (optional)
 	PString* callSigAdr,
+	/// bind IP for BindAndRouteToGateway
+	PString* bindIP,
 	/// set by the function to true if another route request for the same
 	/// call is pending
 	bool& duplicate
@@ -920,7 +980,7 @@ VirtualQueue::RouteRequest* VirtualQueue::InsertRequest(
 	}
 
 	// insert the new pending route request
-	RouteRequest* r = new RouteRequest(callingEpId, crv, callID, agent, callSigAdr);
+	RouteRequest* r = new RouteRequest(callingEpId, crv, callID, agent, callSigAdr, bindIP);
 	m_pendingRequests.push_back(r);
 	return r;
 }
@@ -972,11 +1032,13 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 			PString source = AsDotString(ep->GetCallSignalAddress());
 			PString epid = ep->GetEndpointIdentifier().GetValue();
 			PString * callSigAdr = new PString();
-			if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, reject, vq, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid)))
+			PString * bindIP = new PString();
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(arq.m_callReferenceValue), aliases, callSigAdr, bindIP, reject, vq, AsString(arq.m_srcInfo), AsString(arq.m_callIdentifier.m_guid)))
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
 			if (reject) {
 				request.SetFlag(RoutingRequest::e_Reject);
 			}
+			request.SetSourceIP(*bindIP);
 			if (!callSigAdr->IsEmpty()) {
 				if (!arq.HasOptionalField(H225_AdmissionRequest::e_destCallSignalAddress)) {
 					arq.IncludeOptionalField(H225_AdmissionRequest::e_destCallSignalAddress);
@@ -989,6 +1051,7 @@ bool VirtualQueuePolicy::OnRequest(AdmissionRequest & request)
 				arq.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 			}
 			delete callSigAdr;
+			delete bindIP;
 		}
 		// the trick: if empty, the request is rejected
 		// so we return true to terminate the routing
@@ -1030,15 +1093,17 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 				epid = "unknown";	// make sure its not empty
 			}
 			PString * callSigAdr = new PString(); /* unused for LRQs */
+			PString * bindIP = new PString();
 			PString sourceInfo = "";
 			if (lrq.HasOptionalField(H225_LocationRequest::e_sourceInfo) && (lrq.m_sourceInfo.GetSize() > 0))
 				sourceInfo = AsString(lrq.m_sourceInfo);
 			PString callID = "-";	/* not available for LRQs */
-			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, reject, vq, sourceInfo, callID))
+			if (m_vqueue->SendRouteRequest(source, epid, unsigned(lrq.m_requestSeqNum), aliases, callSigAdr, bindIP, reject, vq, sourceInfo, callID))
 				request.SetFlag(RoutingRequest::e_aliasesChanged);
 			if (reject) {
 				request.SetFlag(RoutingRequest::e_Reject);
 			}
+			request.SetSourceIP(*bindIP);
 			if (!reject && !callSigAdr->IsEmpty()) {
 				// 'explicit' policy can't handle LRQs, so we do it directly
 				PStringArray adr_parts = callSigAdr->Tokenise(":", FALSE);
@@ -1052,9 +1117,11 @@ bool VirtualQueuePolicy::OnRequest(LocationRequest & request)
 				);
 				request.AddRoute(route);					
 				delete callSigAdr;
+				delete bindIP;
 				return true;	// stop processing
 			}
 			delete callSigAdr;
+			delete bindIP;
 			// the trick: if empty, the request is rejected
 			// so we return true to terminate the routing
 			// decision process, otherwise the aliases is
@@ -1082,6 +1149,7 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 		PString epid = "unregistered";
 		const unsigned crv = request.GetWrapper()->GetCallReference();
 		PString * callSigAdr = new PString();
+		PString * bindIP = new PString();
 		PString callid = AsString(setup.m_callIdentifier.m_guid);
 		PString src = AsString(setup.m_sourceAddress);
 		PIPSocket::Address localAddr;
@@ -1093,12 +1161,13 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 			<< crv << ')'
 			);
 
-		if (m_vqueue->SendRouteRequest(callerip, epid, crv, aliases, callSigAdr, reject, vq, src, callid, calledIP))
+		if (m_vqueue->SendRouteRequest(callerip, epid, crv, aliases, callSigAdr, bindIP, reject, vq, src, callid, calledIP))
 			request.SetFlag(RoutingRequest::e_aliasesChanged);
 		
 		if (reject) {
 			request.SetFlag(RoutingRequest::e_Reject);
 		}
+		request.SetSourceIP(*bindIP);
 		if (!reject && callSigAdr->IsEmpty()) {
 			if (!setup.HasOptionalField(H225_Setup_UUIE::e_destinationAddress)) {
 				setup.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
@@ -1121,6 +1190,7 @@ bool VirtualQueuePolicy::OnRequest(SetupRequest & request)
 			setup.m_destCallSignalAddress = SocketToH225TransportAddr(ip, port);
 		}
 		delete callSigAdr;
+		delete bindIP;
 		// the trick: if empty, the request is rejected
 		// so we return true to terminate the routing
 		// decision process, otherwise the aliases is
@@ -1330,7 +1400,7 @@ bool ENUMPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_Ali
 
 class DestinationRoutes {
 public:
-	DestinationRoutes() { m_endChain = false; m_reject = false; m_aliasesChanged = false;}
+	DestinationRoutes() { m_endChain = false; m_reject = false; m_rejectReason = 0; m_aliasesChanged = false; }
 	~DestinationRoutes() { }
 	
 	bool EndPolicyChain() const { return m_endChain; }
@@ -1376,6 +1446,7 @@ protected:
 		const PString & callingStationId,
 		const PString & callid,
 		const PString & messageType,
+		const PString & clientauthid,
 		/* out: */
 		DestinationRoutes & destination);
 
@@ -1460,9 +1531,10 @@ bool SqlPolicy::OnRequest(AdmissionRequest & request)
 		PString callingStationId = request.GetCallingStationId();
 		PString callid = AsString(arq.m_callIdentifier.m_guid);
 		PString messageType = "ARQ";
+		PString clientauthid = request.GetClientAuthId();
 		DestinationRoutes destination;
 
-		DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType,
+		DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
 						/* out: */ destination);
 
 		if (destination.RejectCall()) {
@@ -1513,9 +1585,10 @@ bool SqlPolicy::OnRequest(LocationRequest & request)
 		callingStationId = caller;
 	PString callid = "";	/* not available for LRQs */
 	PString messageType = "LRQ";
+	PString clientauthid = "";	/* not available for LRQs */
 	DestinationRoutes destination;
 
-	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId,callid, messageType,
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId,callid, messageType, clientauthid,
 					/* out: */ destination);
 
 	if (destination.RejectCall()) {
@@ -1560,9 +1633,10 @@ bool SqlPolicy::OnRequest(SetupRequest & request)
 	PString callingStationId = request.GetCallingStationId();
 	PString callid = AsString(setup.m_callIdentifier.m_guid);
 	PString messageType = "Setup";
+	PString clientauthid = request.GetClientAuthId();
 	DestinationRoutes destination;
 
-	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType,
+	DatabaseLookup(	/* in */ source, calledAlias, calledIP, caller, callingStationId, callid, messageType, clientauthid,
 					/* out: */ destination);
 
 	if (destination.RejectCall()) {
@@ -1605,6 +1679,7 @@ void SqlPolicy::DatabaseLookup(
 		const PString & callingStationId,
 		const PString & callid,
 		const PString & messageType,
+		const PString & clientauthid,
 		/* out: */
 		DestinationRoutes & destination)
 {
@@ -1618,6 +1693,7 @@ void SqlPolicy::DatabaseLookup(
 	params["Calling-Station-Id"] = callingStationId;
 	params["i"] = callid;
 	params["m"] = messageType;
+	params["client-auth-id"] = clientauthid;
 	GkSQLResult* result = m_sqlConn->ExecuteQuery(m_query, params, m_timeout);
 	if (result == NULL) {
 		PTRACE(2, m_name << ": query failed - timeout or fatal error");
@@ -1676,7 +1752,7 @@ void SqlPolicy::DatabaseLookup(
 		PString newDestinationIP = resultRow[1].first;
 		PTRACE(5, m_name << "\tQuery result : " << newDestinationAlias << ", " << newDestinationIP);
 		if (newDestinationAlias.ToUpper() == "REJECT") {
-			destination.SetRejectCall(true);;
+			destination.SetRejectCall(true);
 			destination.SetRejectReason(newDestinationIP.AsInteger());
 		} else {
 			H225_ArrayOf_AliasAddress newAliases;
@@ -1712,6 +1788,63 @@ void SqlPolicy::DatabaseLookup(
 }
 
 
+// a policy to route all calls to one default endpoint
+class CatchAllPolicy : public Policy {
+public:
+	CatchAllPolicy();
+	virtual ~CatchAllPolicy() { }
+
+protected:
+	virtual bool OnRequest(AdmissionRequest & request) { return CatchAllRoute(request); }
+	virtual bool OnRequest(LocationRequest & request) { return CatchAllRoute(request); }
+	virtual bool OnRequest(SetupRequest & request) { return CatchAllRoute(request); }
+
+	bool CatchAllRoute(RoutingRequest & request) const;
+	
+	PString m_catchAllAlias;
+	PString m_catchAllIP;
+};
+
+CatchAllPolicy::CatchAllPolicy()
+{
+	m_name = "CatchAllPolicy";
+	static const char *defaultPolicySection = "Routing::CatchAll";
+	PConfig* cfg = GkConfig();
+	m_catchAllAlias = cfg->GetString(defaultPolicySection, "CatchAllAlias", "catchall");
+	m_catchAllIP = cfg->GetString(defaultPolicySection, "CatchAllIP", "");
+	if (!m_catchAllIP.IsEmpty() && (m_catchAllIP.Find(':') == P_MAX_INDEX)) {
+		m_catchAllIP += ":1720";	// assume default port if none specified
+	}
+}
+
+bool CatchAllPolicy::CatchAllRoute(RoutingRequest & request) const
+{
+	if (!m_catchAllIP.IsEmpty()) {
+		H225_TransportAddress destAddr;
+		if (GetTransportAddress(m_catchAllIP, GK_DEF_UNICAST_RAS_PORT, destAddr)) {
+			Route route("catchall", destAddr);
+			route.m_destEndpoint = RegistrationTable::Instance()->FindBySignalAdr(route.m_destAddr);
+			request.AddRoute(route);
+			return true;
+		} else {
+			PTRACE(1, m_name << "\tInvalid catch-all IP " << m_catchAllIP);
+		}
+	}
+	H225_ArrayOf_AliasAddress find_aliases;
+	find_aliases.SetSize(1);
+	H323SetAliasAddress(m_catchAllAlias, find_aliases[0]);
+	endptr ep = RegistrationTable::Instance()->FindByAliases(find_aliases);
+	if (ep) {
+		Route route(ep);
+		route.m_policy = m_name;
+		request.AddRoute(route);
+		return true;
+	}
+	PTRACE(1, m_name << "\tCatch-all endpoint " << m_catchAllAlias << " not found!");
+	return false;	// configured default endpoint not found
+}
+
+
 namespace { // anonymous namespace
 	SimpleCreator<ExplicitPolicy> ExplicitPolicyCreator("explicit");
 	SimpleCreator<InternalPolicy> InternalPolicyCreator("internal");
@@ -1721,6 +1854,7 @@ namespace { // anonymous namespace
 	SimpleCreator<NumberAnalysisPolicy> NumberAnalysisPolicyCreator("numberanalysis");
 	SimpleCreator<ENUMPolicy> ENUMPolicyCreator("enum");
 	SimpleCreator<SqlPolicy> SqlPolicyCreator("sql");
+	SimpleCreator<CatchAllPolicy> CatchAllPolicyCreator("catchall");
 }
 
 
