@@ -12,7 +12,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef GKAUTH_H
-#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.45 2009/05/24 21:53:29 willamowius Exp $"
+#define GKAUTH_H "@(#) $Id: gkauth.h,v 1.49 2010/01/24 21:04:46 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -123,6 +123,8 @@ struct ARQAuthData
 	int m_proxyMode;
 	/// RADIUS Class attribute, if found in Access-Accept/Access-Reject
 	PBYTEArray m_radiusClass;
+	/// ID provided by client, to be passed out with accounting events
+	PUInt64 m_clientAuthId;
 	
 private:
 	ARQAuthData();
@@ -175,6 +177,8 @@ struct SetupAuthData
 	int m_proxyMode;
 	/// RADIUS Class attribute, if found in Access-Accept/Access-Reject
 	PBYTEArray m_radiusClass;
+	/// ID provided by client, to be passed out with accounting events
+	PUInt64 m_clientAuthId;
 		
 private:
 	SetupAuthData();
@@ -316,7 +320,7 @@ public:
 	virtual int Check(RasPDU<H225_LocationRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_InfoRequest>& req, unsigned& rejectReason);
 
-	/** Authenticate/Authorize RAS or signalling message.
+	/** Authenticate/Authorize RAS or signaling message.
 	
 	    @return
 	    e_fail - authentication rejected the request
@@ -550,7 +554,7 @@ public:
 	virtual int Check(RasPDU<H225_LocationRequest>& req, unsigned& rejectReason);
 	virtual int Check(RasPDU<H225_InfoRequest>& req, unsigned& rejectReason);
 
-	/** Authenticate/Authorize RAS or signalling message. 
+	/** Authenticate/Authorize RAS or signaling message. 
 	    An override from GkAuthenticator.
 	
 	    @return
@@ -560,7 +564,7 @@ public:
 	             or cannot be determined (SQL failure, no cryptoTokens, ...)
 	*/
 	virtual int Check(
-		/// ARQ to be authenticated/authorized
+		/// RRQ to be authenticated/authorized
 		RasPDU<H225_RegistrationRequest>& request, 
 		/// authorization data (reject reason, ...)
 		RRQAuthData& authData
@@ -756,7 +760,7 @@ private:
 
 #ifdef H323_H350
 
-/// Generic SQL authenticator for H.235 enabled endpoints
+/// H.350 authenticator for H.235 enabled endpoints
 class H350PasswordAuth : public SimplePasswordAuth
 {
 public:
@@ -810,33 +814,33 @@ public:
 	
 	/// an override from GkAuthenticator
 	virtual int Check(
-		/// ARQ to be authenticated/authorized
+		/// RRQ to be authenticated/authorized
 		RasPDU<H225_RegistrationRequest>& request, 
 		/// authorization data (reject reason, ...)
 		RRQAuthData& authData
 		);
 
 protected:
-	/** Validate that the signalling addresses match the given condition.
+	/** Validate that the signaling addresses match the given condition.
 	    The condition consists of one or more auth rules.
 		
 	    @return
-	    true if the signalling addresses match the condition.
+	    true if the signaling addresses match the condition.
 	*/
 	virtual bool doCheck(
-		/// an array of source signalling addresses for an endpoint that sent the request
+		/// an array of source signaling addresses for an endpoint that sent the request
 		const H225_ArrayOf_TransportAddress& sigaddr,
 		/// auth condition string as returned by GetAuthConditionString
 		const PString& condition
 		);
 		
-	/** Validate that the signalling address matches the given auth rule.
+	/** Validate that the signaling address matches the given auth rule.
 	
 	    @return
 	    true if the signal address matches the rule.
 	*/
 	virtual bool CheckAuthRule(
-		/// a signalling address for the endpoint that sent the request
+		/// a signaling address for the endpoint that sent the request
 		const H225_TransportAddress& sigaddr,
 		/// the auth rule to be used for checking
 		const PString& authrule
