@@ -11,6 +11,27 @@
  * with the OpenH323 library.
  *
  * $Log: radacct.cxx,v $
+ * Revision 1.37  2009/11/15 15:45:52  willamowius
+ * code cleanup
+ *
+ * Revision 1.36  2009/10/16 11:41:47  willamowius
+ * set Radius VSA attribute h323_rtp_proxy in stop event
+ *
+ * Revision 1.35  2009/09/27 13:30:09  shorne
+ * Fix for previous commit
+ *
+ * Revision 1.34  2009/09/26 02:26:55  shorne
+ * Fix compile warning in Windows
+ *
+ * Revision 1.33  2009/09/11 20:21:51  willamowius
+ * new Radius attributes on stop: h323pddtime, h323ringtime, h323routeattempts thanks Yuriy Georgiewskiy
+ *
+ * Revision 1.32  2009/09/09 07:33:50  willamowius
+ * whitespace change
+ *
+ * Revision 1.31  2009/06/20 10:02:24  willamowius
+ * send RTCP statistics to Radius server (thanks to Georgiewskiy Yuriy)
+ *
  * Revision 1.30  2009/05/24 22:38:51  willamowius
  * everything that holds a time should be a time_t, not a long
  *
@@ -229,7 +250,6 @@ GkAcctLogger::Status RadAcct::Log(
 		))));
 
 	PIPSocket::Address addr;
-	WORD port;
 					
 	// Gk works as NAS point, so append NAS IP
 	pdu->AppendAttr(RadiusAttr::NasIpAddress, m_nasIpAddress);
@@ -320,9 +340,87 @@ GkAcctLogger::Status RadAcct::Log(
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_release_source,call->GetReleaseSource());
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_preferred_codec,call->GetCodec());
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_rewritten_e164_num,call->GetCalledStationId());
+
+				// Post Dial Delay Time
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("h323pddtime=")+PString(PString::Unsigned,(long)call->GetPostDialDelay()),true
+				);
+
+				// Ring Time
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("h323ringtime=")+PString(PString::Unsigned,(long)call->GetRingTime()),true
+				);
+
+				// Number of Route Attempts
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("h323routeattempts=")+PString(PString::Unsigned,call->GetNoCallAttempts()),true
+				);
+				
+				// Proxy Mode
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("h323_rtp_proxy=")+PString(PString::Unsigned,
+					((call->GetProxyMode() == CallRec::ProxyEnabled) ? 1 : 0)),true
+				);
+
+				//RTCP SOURCE REPORT
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTP_source_IP=")+call->GetSRC_media_IP(),
+    				    true
+				);
+				
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTP_destination_IP=")+call->GetDST_media_IP(),
+    				    true
+				);
+				
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_source_packet_count=")+PString(PString::Unsigned,call->GetRTCP_SRC_packet_count()),
+    				    true
+				);
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_source_packet_lost=")+PString(PString::Unsigned,call->GetRTCP_SRC_packet_lost()),
+				    true
+				);
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_source_jitter=")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_min())+PString("|")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_avg())+PString("|")+PString(PString::Unsigned,call->GetRTCP_SRC_jitter_max()),
+				    true
+				);
+				
+				PINDEX i_sdes = 0;
+				PStringList sdes = call->GetRTCP_SRC_sdes();
+				while (i_sdes < sdes.GetSize()) {
+				    pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("RTCP_source_sdes_")+sdes[i_sdes],
+					true
+				    );
+				    i_sdes ++;
+				}
+				//RTCP DESTINATION REPORT
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_destination_packet_count=")+PString(PString::Unsigned,call->GetRTCP_DST_packet_count()),
+				    true
+				);
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_destination_packet_lost=")+PString(PString::Unsigned,call->GetRTCP_DST_packet_lost()),
+				    true
+				);
+				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+				    PString("RTCP_destination_jitter=")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_min())+PString("|")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_avg())+PString("|")+PString(PString::Unsigned,call->GetRTCP_DST_jitter_max()),
+				    true
+				);
+				i_sdes = 0;
+				sdes = call->GetRTCP_DST_sdes();
+				while (i_sdes < sdes.GetSize()) {
+				    pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_AV_Pair,
+					PString("RTCP_destination_sdes_")+sdes[i_sdes],
+					true
+				    );
+				    i_sdes ++;
+				}
 			}					
-			
-			if (call->GetDestSignalAddr(addr,port))
+
+			WORD port;
+			if (call->GetDestSignalAddr(addr, port))
 				pdu->AppendCiscoAttr(RadiusAttr::CiscoVSA_h323_remote_address,
 					addr.AsString()
 					);
@@ -332,6 +430,7 @@ GkAcctLogger::Status RadAcct::Log(
 					+ GetGUIDString(call->GetCallIdentifier().m_guid),
 				true
 				);
+			
 		}
 	
 		pdu->AppendAttr(RadiusAttr::AcctDelayTime, 0);
