@@ -9,7 +9,7 @@
 //
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
-// We also explicitely grant the right to link this code
+// We also explicitly grant the right to link this code
 // with the OpenH323 library.
 //
 // initial author: Chih-Wei Huang <cwhuang@linux.org.tw>
@@ -18,7 +18,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef RASINFO_H
-#define RASINFO_H "@(#) $Id: rasinfo.h,v 1.4 2007/09/27 21:15:34 willamowius Exp $"
+#define RASINFO_H "@(#) $Id: rasinfo.h,v 1.6 2009/04/20 09:28:49 willamowius Exp $"
 
 // define a type for an RAS tag
 template<int I> struct RasTag {
@@ -202,7 +202,10 @@ template<> struct RasInfo<H225_ResourcesAvailableIndicate> : public TagInfo<H225
 template<> struct RasInfo<H225_ResourcesAvailableConfirm> : public TagInfo<H225_RasMessage::e_resourcesAvailableConfirm> {};
 template<> struct RasInfo<H225_InfoRequestAck> : public TagInfo<H225_RasMessage::e_infoRequestAck> {};
 template<> struct RasInfo<H225_InfoRequestNak> : public TagInfo<H225_RasMessage::e_infoRequestNak> {};
-template<> struct RasInfo<H225_ServiceControlIndication> : public TagInfo<H225_RasMessage::e_serviceControlIndication> {};
+template<> struct RasInfo<H225_ServiceControlIndication> : public TagInfo<H225_RasMessage::e_serviceControlIndication> {
+	typedef RasTag<tag+1> ConfirmTag;
+	typedef RasType<tag+1>::Type ConfirmType;
+};
 template<> struct RasInfo<H225_ServiceControlResponse> : public TagInfo<H225_RasMessage::e_serviceControlResponse> {};
 
 #endif // RASINFO_H
