@@ -11,6 +11,9 @@
  * with the OpenH323 library.
  *
  * $Log: statusacct.h,v $
+ * Revision 1.5  2009/12/02 14:32:13  willamowius
+ * add new accounting events: AcctAlert, AcctRegister, AcctUnregister
+ *
  * Revision 1.4  2009/02/09 13:25:59  willamowius
  * typo in comment
  *
@@ -33,7 +36,7 @@
  */
 
 #ifndef __STATUSACCT_H
-#define __STATUSACCT_H "@(#) $Id: statusacct.h,v 1.4 2009/02/09 13:25:59 willamowius Exp $"
+#define __STATUSACCT_H "@(#) $Id: statusacct.h,v 1.5 2009/12/02 14:32:13 willamowius Exp $"
 
 #include "gkacct.h"
 
@@ -48,7 +51,7 @@ public:
 	enum Constants
 	{
 		/// events recognized by this module
-		StatusAcctEvents = AcctStart | AcctStop | AcctUpdate | AcctConnect
+		StatusAcctEvents = AcctStart | AcctStop | AcctUpdate | AcctConnect | AcctAlert | AcctRegister | AcctUnregister
 	};
 
 	StatusAcct( 
@@ -68,6 +71,12 @@ public:
 		const callptr& call
 		);
 
+	/// overriden from GkAcctLogger
+	virtual Status Log(
+		AcctEvent evt,
+		const endptr& ep
+		);
+
 	/// overriden from GkAcctLogger
 	virtual PString EscapeAcctParam(const PString& param) const;
 
@@ -95,6 +104,12 @@ private:
 	PString m_updateEvent;
 	/// parametrized string for the call connect event
 	PString m_connectEvent;
+	/// parametrized string for the call alerting event
+	PString m_alertEvent;
+	/// parametrized string for the endpoint register event
+	PString m_registerEvent;
+	/// parametrized string for the endpoint un-register event
+	PString m_unregisterEvent;
 	/// timestamp formatting string
 	PString m_timestampFormat;
 };
